package com.sensor.iot.api.event;

import com.sensor.iot.api.model.ApiResp;
import com.sensor.iot.api.model.ExecFuncReq;
import com.sensor.iot.api.model.ReadPropReq;
import com.sensor.iot.api.model.WritePropReq;
import com.sensor.iot.api.service.DeviceOptApiService;
import lombok.RequiredArgsConstructor;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.Optional;

/**
 * AsyncRequestListener
 *
 * @author Frank
 */
@Component
@RequiredArgsConstructor
public class AsyncRequestListener {

    private final Optional<IAsyncResponseListener> asyncResponseListener;
    private final DeviceOptApiService deviceOptApiService;

    @EventListener(WritePropReq.class)
    public void onUpdateProp(WritePropReq params) {
        ApiResp<Map<String, Object>> resp = deviceOptApiService.writeProperty(params);
        asyncResponseListener.ifPresent(response -> response.onResponse(resp));
    }

    @EventListener(ExecFuncReq.class)
    public void onFuncInvoke(ExecFuncReq params) {
        ApiResp<Map<String, Object>> resp = deviceOptApiService.funcInvoke(params);
        asyncResponseListener.ifPresent(response -> response.onResponse(resp));
    }

    @EventListener(ReadPropReq.class)
    public void onReadProp(ReadPropReq params) {
        ApiResp<Map<String, Object>> resp = deviceOptApiService.readProperty(params);
        asyncResponseListener.ifPresent(response -> response.onResponse(resp));
    }
}
